"use strict";
/*!
 * Copyright (c) 2021-present, Okta, Inc. and/or its affiliates. All rights reserved.
 * The Okta software accompanied by this notice is provided pursuant to the Apache License, Version 2.0 (the "License.")
 *
 * You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0.
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and limitations under the License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const path_1 = require("path");
const compileFactory_1 = __importDefault(require("./compileFactory"));
const normalizeOpts_1 = require("./normalizeOpts");
const nodes_1 = require("./nodes");
function plugin({ types: t }) {
    const compile = (0, compileFactory_1.default)();
    return {
        name: "odyssey",
        visitor: {
            ImportDeclaration(path, state) {
                var _a, _b;
                const opts = (0, normalizeOpts_1.normalizeOpts)(state.opts);
                const importer = (_b = (_a = state === null || state === void 0 ? void 0 : state.file) === null || _a === void 0 ? void 0 : _a.opts) === null || _b === void 0 ? void 0 : _b.filename;
                if (typeof importer !== "string") {
                    return;
                }
                const specifier = path.node.specifiers[0];
                if (!t.isImportDefaultSpecifier(specifier)) {
                    return;
                }
                const importee = path.node.source.value;
                if (!(0, normalizeOpts_1.shouldInclude)(importee, opts.include)) {
                    return;
                }
                if (opts.identityObjectProxy) {
                    path.replaceWith((0, nodes_1.identityObjectProxyVariableDeclaration)({
                        name: specifier.local.name,
                    }));
                    return;
                }
                const filePath = (0, path_1.resolve)((0, path_1.dirname)(importer), importee);
                const file = compile({ filePath });
                path.replaceWith((0, nodes_1.variableDeclaration)({
                    name: specifier.local.name,
                    ...file,
                }));
            },
            CallExpression(path, state) {
                var _a, _b;
                const opts = (0, normalizeOpts_1.normalizeOpts)(state.opts);
                const importer = (_b = (_a = state === null || state === void 0 ? void 0 : state.file) === null || _a === void 0 ? void 0 : _a.opts) === null || _b === void 0 ? void 0 : _b.filename;
                if (typeof importer !== "string") {
                    return;
                }
                const callee = path.node.callee;
                if (!t.isIdentifier(callee)) {
                    return;
                }
                if (callee.name !== "require") {
                    return;
                }
                const argument = path.node.arguments[0];
                if (!t.isStringLiteral(argument)) {
                    return;
                }
                const importee = argument.value;
                if (!(0, normalizeOpts_1.shouldInclude)(importee, opts.include)) {
                    return;
                }
                if (opts.identityObjectProxy) {
                    path.replaceWith((0, nodes_1.identityObjectProxy)());
                    return;
                }
                const filePath = (0, path_1.resolve)((0, path_1.dirname)(importer), importee);
                const file = compile({ filePath });
                path.replaceWith((0, nodes_1.tokenObjectExpression)(file));
            },
        },
    };
}
exports.plugin = plugin;
